// Copyright (c) 2005 Nicolas P. Rougier (rougier@loria.fr)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Persistence of Vision Ray Tracer Scene Description File
// File: chain.pov
// Vers: 3.5
// Desc: 
// Date: 06/2005
// Auth: Nicolas Rougier
// Cmd : povray +Ichain.pov +w1200 +h900 +fn +am2 +a0.1 +P
// ---------------------------------------------------------------------------
#version 3.5;
#include "colors.inc"
#include "functions.inc"

// ========================================
//  Switches
// ========================================
#declare RadOK				= 2;
#declare use_chain			= 1;
#declare use_iso_chain		= 1;
#declare iso_chain_acc		= 0.001;
#declare iso_chain_maxgrad	= 2.0;
#declare use_hf_wall		= 1;
#declare hf_wall_resolution	= 1000;
#declare use_blur			= 1;
#declare use_tex			= 1;
#declare Seed				= seed (0);


// ========================================
//  Settings
// ========================================
#default {finish {ambient 0}}
global_settings {
  #if (RadOK > 0)
    radiosity {
      pretrace_start 0.08
      pretrace_end   0.01
      #if (RadOK=2)
        count 250 error_bound 0.5
      #else
        count 35 error_bound 1.8
      #end
      nearest_count 7
      recursion_limit 1
      low_error_factor .75 
      gray_threshold 0  
      minimum_reuse 0.015
      brightness 1
      adc_bailout 0.01/2
    }
  #end  
}

// ========================================
//  Ground
// ========================================
plane {y, -100 pigment{rgb 1} hollow}


// ========================================
//  Sky
// ========================================
#declare C_Sky=rgb <90,131,255>/255;
sky_sphere {
  pigment {
    gradient y
    color_map {
      [0 C_Sky*0.05]
      [0.4 C_Sky]
    }
  }
  scale 2
  translate -y
}

// ========================================
//  Camera
// ========================================
#declare EyePos  = <-0.5, 0.75, -4.0>;
#declare EyeLook = < 0.9, 0.1,  0.0>;
camera {
  location EyePos
  look_at  EyeLook
  direction 1.5*z
  right     x*image_width/image_height
  angle 40
  #if (use_blur)
	focal_point <0.55,  0.3, -1.25>
	aperture 0.05
	blur_samples 75
	variance 1/10000
  #end
}

// ========================================
//  Sun
// ========================================
//#declare C_Sun=rgb <248,226,192>/255;
#declare C_Sun=rgb <248,236,222>/255;
light_source {
  10000*y  C_Sun*2.5
//  area_light <500, 0, 0> <0, 0, 500> 3,3 jitter circular orient
  rotate -x*60 rotate  y*85
}

// ========================================
//  Link macros (iso or csg)
// ========================================
#declare major  = 0.7;
#declare minor  = 0.3;
#declare length = 1.00;

#declare f_holes = function {
  pigment {
	bumps
    color_map {
      [0.01 rgb 1.00]
      [0.40 rgb 0.00]
	}
	scale .02
  }
}

#declare f_micro_holes = function {
  pigment {
	bumps
    color_map {
      [0.25 rgb 1.00]
      [0.75 rgb 0.00]
	}
	scale .0075 //0.005
  }
}

#declare f_bump = function {
  pigment {
	bumps turbulence .25
    color_map {
	  [0.10 rgb 1.00]
      [0.90 rgb 0.00]    
	}
	scale .15
  }
}
#declare f_granite2 = function {
  pigment {
	granite
    color_map {
      [0.0 rgb 1.00]
      [1.0  rgb 0.00]
	}
	scale .25
  }
}

#declare f_rust = function {
//    0.0015*f_micro_holes(x,y,z).grey
  + 0.0050*f_holes(x,y,z).grey
  - 0.0250*f_bump(x,y,z).grey
  + 0.0050*f_granite2(x,y,z).grey

}

#macro link ()
  #local r = 10*rand(Seed);
  #if (use_iso_chain)
	union {
	  isosurface {
		function {
		  f_torus(x,y,z, major, minor) + f_rust (x+r,y+r,z+r)
		}
		accuracy iso_chain_acc
		max_gradient iso_chain_maxgrad
		contained_by {box {<-2,2,-2> <0,-2,2>}}
	  }  

	  isosurface {
		function {sqrt(y*y+z*z)-minor + f_rust (x+r,y+r,z+r+major)}
		accuracy iso_chain_acc
		max_gradient iso_chain_maxgrad
		contained_by {box {<0,2,-2> <length,-2,2>}}
		translate z*major
	  }  
	  isosurface {
		function {sqrt(y*y+z*z)-minor + f_rust (x+r,y+r,z+r-major)}
		accuracy iso_chain_acc
		max_gradient iso_chain_maxgrad
		contained_by {box {<0,2,-2> <length,-2,2>}}
		translate -z*major
	  }  
	  
	  isosurface {
		function {f_torus(x,y,z, major, minor) + f_rust (x+r+length,y+r,z+r)
		}
		accuracy iso_chain_acc
		max_gradient iso_chain_maxgrad
		contained_by {box {<0,2,-2> <2,-2,2>}}
		translate x*length
	  }
	  translate -x*length/2 
	}
  #else
	union {
	  intersection {
		torus {major, minor}
		box {<-1,-1,-1> <0,1,1>}
	  }
	  intersection {
		torus {major, minor}
		box {<0,-1,-1> <1,1,1>}
		translate length*x
	  }
	  cylinder {
		<0, 0, -major>,<length,0, -major>, minor
	  }
	  cylinder {
		<0, 0, major>,<length,0, major>, minor
	  }
	  translate -x*length/2 
	}
  #end
#end


// ========================================
//  Chain
// ========================================
#declare p_rust = pigment { 
  image_map{jpeg "rust.jpg"}
  translate -.5
  scale 10
  warp {turbulence 1 lambda 2}
}
#declare p_brass = pigment { 
  image_map{jpeg "iron06.jpg"}
  translate -.5
  scale 10
  warp {turbulence 1 lambda 2}
}

#declare rusted_metal = texture {
  #if (use_tex)
	pigment { 
	  cells
	  warp {turbulence 1 lambda 3}
	  pigment_map {
		[0.7 p_rust]
		[1.0 p_brass]
	  }
	}
	#if (use_iso_chain = 0)
	  normal {granite .1 scale .1}
	#end
	//  finish {metallic specular 1.0 roughness 0.01}
//    finish {specular 0 roughness 0.01 diffuse .75 ambient .0}
    finish {specular .25 roughness 0.01 diffuse .75 ambient 0}
  #else
	pigment {rgb 1}
  #end
}


#if (use_chain)
  #declare link_1 = object {link() rotate x*90  rotate y*35 translate <0,-.3,0>}
  #declare link_2 = object {link() rotate -y*10 rotate x*40 translate <1.6*length,-.15,-.5>}
  #declare link_3 = object {link() rotate -y*90 rotate -z*55 rotate -x*10 translate <-0.67*length,.35,1.2>}
  #declare link_4 = object {link() rotate x*90  rotate -z*60 rotate y*40 translate <0.1*length,1.3,1.5>}
  #declare link_5 = object {link() rotate x*80  rotate y*50  rotate -z*30 rotate y*50 translate <1.9,-.95,-.45>}
  union {
    object {link_1 texture {rusted_metal translate <1,2,3>*29.8}}
    object {link_2 texture {rusted_metal translate <1,2,3>*29.95}}
    object {link_3 texture {rusted_metal translate <1,2,3>*30}}
    object {link_4 texture {rusted_metal translate <1,2,3>*30.1}}
    object {link_5 texture {rusted_metal translate <1,2,3>*30.2}}
  }
#end



// ========================================
//  Wall
// ========================================
#declare p_map  = pigment {image_map {jpeg "im_concrete-01.jpg"}}
#declare p_mapf = pigment {image_map {jpeg "im_concrete-01.jpg" transmit all .75}}
#declare t_wall = texture {
  pigment {
	p_map
	scale 100 warp{turbulence .3 lambda 3} scale .01
	translate <-.5,0,0> scale 5
	translate .5*y
  }
}
texture {
  pigment {
	p_mapf
	warp {turbulence 1 lambda 3}
	scale 100 warp{turbulence .3 lambda 3} scale .01
  }
  scale 5
  translate .5*y
}

#if (use_hf_wall)
  height_field {
	function hf_wall_resolution,hf_wall_resolution {.085*f_granite(10+x*3,20+y*3,30+z*3)}
	translate <-.5,0,-.5>
	scale <6, 3, 6>
	rotate x*90
	translate -z*3
	texture {t_wall translate -y*1.95}
	rotate y*35
	translate <3.5, 0, 3.25>
    hollow
  }
#else
  box {
    -3,3
	texture {t_wall scale <1,-1,1> translate -y*1}
	rotate y*35
	translate <3.5, 0, 3.25>
    hollow
  }
#end	
